<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Http;

class MainController extends Controller
{

    // Function to calculate distance using the Haversine formula
    private function haversineGreatCircleDistance($latitudeFrom, $longitudeFrom, $latitudeTo, $longitudeTo, $earthRadius = 6371) {
        $latFrom = deg2rad($latitudeFrom);
        $lonFrom = deg2rad($longitudeFrom);
        $latTo = deg2rad($latitudeTo);
        $lonTo = deg2rad($longitudeTo);

        $latDelta = $latTo - $latFrom;
        $lonDelta = $lonTo - $lonFrom;

        $angle = 2 * asin(sqrt(pow(sin($latDelta / 2), 2) +
            cos($latFrom) * cos($latTo) * pow(sin($lonDelta / 2), 2)));
        return $angle * $earthRadius; // Distance in km
    }

    // Function to geocode address into lat/lon
    private function geocodeAddress($address) {
        $apiKey = "32d9575c8f1b4a919c194218f09b987f"; // Use environment variable for the API key
        $address = urlencode($address);
        $url = "https://api.opencagedata.com/geocode/v1/json?q={$address}&key={$apiKey}&limit=1";

        $response = Http::get($url);
        $json = $response->json();

        if (!empty($json['results'])) {
            $latitude = $json['results'][0]['geometry']['lat'];
            $longitude = $json['results'][0]['geometry']['lng'];
            return ['lat' => $latitude, 'lng' => $longitude];
        } else {
            return false; // Geocoding failed
        }
    }

    // Function to reverse geocode lat/lon into an address (excluding street-level details)
private function reverseGeocode($latitude, $longitude) {
    $apiKey = "32d9575c8f1b4a919c194218f09b987f"; // Use environment variable for the API key
    $url = "https://api.opencagedata.com/geocode/v1/json?q={$latitude},{$longitude}&key={$apiKey}&limit=1";

    $response = Http::get($url);
    $json = $response->json();

    if (!empty($json['results'])) {
        // Extract components except street-level details
        $components = $json['results'][0]['components'];
        
        $city = $components['city'] ?? $components['town'] ?? $components['village'] ?? '';
        $state = $components['state'] ?? '';
        $country = $components['country'] ?? '';

        // Combine and return the components (excluding street)
        return trim("$city, $state, $country");
    } else {
        return false; // Reverse geocoding failed
    }
}


    // Method to get nearby agencies
    public function getNearbyAgencies(Request $request) {
        // Validate request parameters
        

        $userLat = $request->input('lat');
        $userLng = $request->input('lng');

        $user_address = $this->reverseGeocode($userLat, $userLng);

        // Fetch agencies from database
        $agencies = DB::table('agencies')->select('id', 'name', 'address', 'phone')->get();

        // Store agencies within 20km
        $nearbyAgencies = [];

        foreach ($agencies as $agency) {
            $agencyAddress = $agency->address;

            // Geocode agency address
            $location = $this->geocodeAddress($agencyAddress);
            if ($location) {
                $agencyLat = $location['lat'];
                $agencyLng = $location['lng'];

                // Calculate distance between user and agency
                $distance = $this->haversineGreatCircleDistance($userLat, $userLng, $agencyLat, $agencyLng);
                
                // Check if the distance is within 20km
                if ($distance <= 80) {
                    $nearbyAgencies[] = $agency;
                }
            }
        }


        return view('directing-master.listing', compact('nearbyAgencies', 'user_address', 'agencies'));
    }
}
